@@ -3,6 +3,7 @@
  *   Copyright 2006-2007 Mick Kappenburg <ksudoku@kappendburg.net>         *
  *   Copyright 2006-2008 Johannes Bergmeier <johannes.bergmeier@gmx.net>   *
  *   Copyright 2012      Ian Wadham <iandw.au@gmail.com>                   *
+ *   Copyright 2015      Ian Wadham <iandw.au@gmail.com>                   *
  *                                                                         *
  *   This program is free software; you can redistribute it and/or modify  *
  *   it under the terms of the GNU General Public License as published by  *
@@ -247,6 +248,8 @@ void KSudoku::startGame(const Game& game) {
 
 	QWidget* widget = view->widget();
 	m_gameUI = view;
+	Game g = currentGame();
+	g.setMessageParent(view->widget());
 
 	wrapper->layout()->addWidget(widget);
 	widget->show();
@@ -265,6 +268,85 @@ void KSudoku::startGame(const Game& game) {
 	m_valueListWidget->setMaxValue(view->game().order());
 	m_valueListWidget->selectValue(1);
 	m_valueListWidget->show();
+
+	SudokuType t = game.puzzle()->graph()->specificType();
+	bool playing = game.puzzle()->hasSolution();
+	if (playing && (t == Mathdoku)) {
+	    KMessageBox::information (this,
+		i18n("Mathdoku puzzles can have any size from 3x3 up to 9x9. "
+		     "The solution is a grid in which every row and every "
+		     "column contains the available digits (1-3 up to 1-9) "
+		     "exactly once. The grid is covered with irregularly "
+		     "shaped cages.\n"
+		     "\n"
+		     "Cages of size 1 are starting-values or clues, but there "
+		     "are not many of them. Cages of larger size have a target "
+		     "value and an arithmetic operator (+-x/). The digits in "
+		     "the cage must combine together, using the operator, to "
+		     "reach the target value, e.g. '12x' means that the digits "
+		     "must multiply together to make 12. A digit can occur "
+		     "more than once in a cage, provided it occurs in "
+		     "different rows and columns.\n"
+		     "\n"
+		     "In general, larger Mathdokus are more difficult and so "
+		     "are larger cages. You can select the puzzle size in "
+		     "KSudoku's Settings dialog and the maximum cage-size by "
+		     "using KSudoku's Difficulty button."),
+		i18n("Playing Mathdoku"), QString("PlayingMathdoku"));
+	}
+	else if (playing && (t == KillerSudoku)) {
+	    KMessageBox::information (this,
+		i18n("Killer Sudoku puzzles can have sizes 4x4 or 9x9, with "
+		     "either four 2x2 blocks or nine 3x3 blocks. The solution "
+		     "must follow Classic Sudoku rules. The difference is that "
+		     "there are few starting-values or clues (if any). Instead "
+		     "the grid is covered with irregularly shaped cages.\n"
+		     "\n"
+		     "Cages of size 1 are starting-values or clues. Cages of "
+		     "larger size have a target value and the digits in them "
+		     "must add up to that value. In Killer Sudoku, a cage "
+		     "cannot contain any digit more than once.\n"
+		     "\n"
+		     "In general, larger cages are more difficult. You can "
+		     "select the maximum cage-size by using KSudoku's "
+		     "Difficulty button."),
+		i18n("Playing Killer Sudoku"), QString("PlayingKillerSudoku"));
+	}
+	else if ((t == Mathdoku) || (t == KillerSudoku)) {
+	    KMessageBox::information (this,
+		i18n("Mathdoku and Killer Sudoku puzzles have to be keyed in "
+		     "by working on one cage at a time. To start a cage, left "
+		     "click on any unused cell or enter a number in the cell "
+		     "that is under the cursor or enter + - / or x there. A "
+		     "small cage-label will appear in that cell. To extend the "
+		     "cage in any direction, left-click on a neigbouring cell "
+		     "or move the cursor there and type a Space.\n"
+		     "\n"
+		     "The number you type is the cage's value and it can have "
+		     "one or more digits, including zero. A cell of size 1 has "
+		     "to have a 1-digit number, as in a normal Sudoku puzzle. "
+		     "It becomes a starting-value or clue for the player.\n"
+		     "\n"
+		     "The + - / or x is the operator (Add, Subtract, Divide or "
+		     "Multiply). You must have one in cages of size 2 or more. "
+		     "In Killer Sudoku, the operator is provided automatically "
+		     "because it is always + or none.\n"
+		     "\n"
+		     "You can enter digits, operators and cells in any order. "
+		     "To complete the cage and start another cage, always "
+		     "press Return. If you make a mistake, the only thing to "
+		     "do is delete a whole cage and re-enter it. Use right "
+		     "click in the current cage or any earlier cage, if you "
+		     "wish to delete it. Alternatively, use the cursor and the "
+		     "Delete or Backspace key.\n"
+		     "\n"
+		     "When the grid is filled with cages, hit the Check "
+		     "button, to solve the puzzle and make sure there is only "
+		     "one solution. If the check fails, you have probably made "
+		     "an error somewhere in one of the cages."),
+		i18n("Data-entry for Puzzles with Cages"),
+		QString("CageDataEntry"));
+	}
 }
 
 void KSudoku::endCurrentGame() {
@@ -335,16 +417,24 @@ void KSudoku::dubPuzzle()
 	int state = puzzle->init(game.allValues());
 
 	if(state <= 0) {
-		KMessageBox::information(this, i18n("Sorry, no solutions have been found."));
+		KMessageBox::information (this,
+		    i18n("Sorry, no solutions have been found. Please check "
+			 "that you have entered in the puzzle completely and "
+			 "correctly."), i18n("Check Puzzle"));
 		delete puzzle;
 		return;
 	} else if(state == 1) {
-		KMessageBox::information(this, i18n("The Puzzle you entered has only one solution."));
+		KMessageBox::information (this,
+		    i18n("The Puzzle you entered has a unique solution and "
+			 "is ready to be played."), i18n("Check Puzzle"));
 	} else {
-		KMessageBox::information(this, i18n("The Puzzle you entered has multiple solutions."));
+		KMessageBox::information (this,
+		    i18n("The Puzzle you entered has multiple solutions. "
+			 "Please check that you have entered in the puzzle "
+			 "completely and correctly."), i18n("Check Puzzle"));
 	}
 
-	if(KMessageBox::questionYesNo(this, i18n("Do you want to play the puzzle now?"), i18n("Play Puzzle"), KGuiItem(i18n("Play")), KStandardGuiItem::cancel() ) == KMessageBox::Yes)
+	if(KMessageBox::questionYesNo(this, i18n("Do you wish to play the puzzle now?"), i18n("Play Puzzle"), KGuiItem(i18n("Play")), KStandardGuiItem::cancel() ) == KMessageBox::Yes)
 	{
 		startGame(ksudoku::Game(puzzle));
 	}
@@ -380,6 +470,10 @@ void KSudoku::setupActions()
 // 	createAction("game_export", SLOT(gameExport()), i18n("Export"));
 
 	KStandardAction::preferences(this, SLOT(optionsPreferences()), actionCollection());
+	// Settings: enable messages that the user marked "Do not show again".
+	KAction* enableMessagesAct = new KAction(i18n("Enable all messages"),0);
+	actionCollection()->addAction("enable_messages", enableMessagesAct);
+	connect(enableMessagesAct, SIGNAL(triggered()), SLOT(enableMessages()));
 
 	//History
 	KStandardGameAction::undo(this, SLOT(undo()), actionCollection());
@@ -632,6 +726,13 @@ void KSudoku::gamePrint()
             i18n("There seems to be no puzzle to print."));
         return;
     }
+    SudokuType t = game.puzzle()->graph()->specificType();
+    if ((t == Mathdoku) || (t == KillerSudoku)) {
+	KMessageBox::information (this,
+	    i18n("Sorry, printing Mathdoku and Killer Sudoku "
+		 "puzzles is not yet supported."));
+	return;
+    }
     sendToPrinter (game.puzzle());
 }
 
@@ -968,6 +1069,17 @@ void KSudoku::loadCustomShapeFromPath()
 	updateShapesList();
 }
 
+void KSudoku::enableMessages()
+{
+	// Enable all messages that the user has marked "Do not show again".
+	int result = KMessageBox::questionYesNo(this,
+					i18n("Enable all messages"));
+	if (result == KMessageBox::Yes) {
+		KMessageBox::enableAllMessages();
+		KGlobal::config()->sync();	// Save the changes to disk.
+	}
+}
+
 #if 0
 KSudokuNewStuff::KSudokuNewStuff( KSudoku* v ) :
         KNewStuff( "ksudoku", (QWidget*) v )
